package main

import (
	"context"
	"flag"
	"log"
	"net/http"
	"os"
	"os/signal"
	"syscall"
	"time"

	"git.autistici.org/id/auth/client"
	"git.autistici.org/id/go-sso/server"
)

var (
	addr       = flag.String("addr", ":4141", "tcp `address` to listen on")
	configPath = flag.String("config", "/etc/sso/server.yml", "configuration `file`")
	authSocket = flag.String("auth-socket", client.DefaultSocketPath, "authentication socket `path`")
)

func main() {
	log.SetFlags(0)
	flag.Parse()

	config, err := server.LoadConfig(*configPath)
	if err != nil {
		log.Fatal(err)
	}

	loginService, err := server.NewLoginService(config)
	if err != nil {
		log.Fatal(err)
	}

	authClient := client.New(*authSocket)
	httpSrv, err := server.New(loginService, authClient, config)
	if err != nil {
		log.Fatal(err)
	}

	srv := &http.Server{
		Addr:         *addr,
		Handler:      httpSrv.Handler(),
		ReadTimeout:  30 * time.Second,
		WriteTimeout: 30 * time.Second,
		IdleTimeout:  60 * time.Second,
	}

	done := make(chan struct{})
	sigCh := make(chan os.Signal, 1)
	go func() {
		<-sigCh
		log.Printf("exiting")

		// Gracefully terminate for 3 seconds max, then shut
		// down remaining clients.
		ctx, cancel := context.WithTimeout(context.Background(), 3*time.Second)
		defer cancel()
		if err := srv.Shutdown(ctx); err == context.Canceled {
			srv.Close()
		}

		close(done)
	}()
	signal.Notify(sigCh, syscall.SIGINT, syscall.SIGTERM)

	if err = srv.ListenAndServe(); err != http.ErrServerClosed {
		log.Fatal("error: %v", err)
	}

	<-done
}
